"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2025.04.10
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import unittest

import agilicus_api
from agilicus_api.api.billing_api import BillingApi  # noqa: E501


class TestBillingApi(unittest.TestCase):
    """BillingApi unit test stubs"""

    def setUp(self):
        self.api = BillingApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_add_billing_usage_record(self):
        """Test case for add_billing_usage_record

        Add usage records for a billing account  # noqa: E501
        """
        pass

    def test_add_customer_balance_transaction(self):
        """Test case for add_customer_balance_transaction

        Add a customer balance transaction  # noqa: E501
        """
        pass

    def test_add_org_to_billing_account(self):
        """Test case for add_org_to_billing_account

        Add an org to a billing account  # noqa: E501
        """
        pass

    def test_add_org_to_billing_subscription(self):
        """Test case for add_org_to_billing_subscription

        Add an org to a billing subscription  # noqa: E501
        """
        pass

    def test_add_subscription_balance_transaction(self):
        """Test case for add_subscription_balance_transaction

        Add a subscription balance transaction  # noqa: E501
        """
        pass

    def test_create_billing_account(self):
        """Test case for create_billing_account

        Create a billing account  # noqa: E501
        """
        pass

    def test_create_feature(self):
        """Test case for create_feature

        create a feature  # noqa: E501
        """
        pass

    def test_create_product(self):
        """Test case for create_product

        Create a product  # noqa: E501
        """
        pass

    def test_create_subscription(self):
        """Test case for create_subscription

        Create a billing subscription  # noqa: E501
        """
        pass

    def test_delete_billing_account(self):
        """Test case for delete_billing_account

        Delete a billing account  # noqa: E501
        """
        pass

    def test_delete_feature(self):
        """Test case for delete_feature

        Delete a feature  # noqa: E501
        """
        pass

    def test_delete_product(self):
        """Test case for delete_product

        Delete a product  # noqa: E501
        """
        pass

    def test_delete_subscription(self):
        """Test case for delete_subscription

        Delete a billing subscription  # noqa: E501
        """
        pass

    def test_get_billing_account(self):
        """Test case for get_billing_account

        Get a single billing account  # noqa: E501
        """
        pass

    def test_get_billing_account_orgs(self):
        """Test case for get_billing_account_orgs

        Get all orgs in a billing account  # noqa: E501
        """
        pass

    def test_get_billing_subscription_orgs(self):
        """Test case for get_billing_subscription_orgs

        Get all orgs in a billing subscription  # noqa: E501
        """
        pass

    def test_get_customer_balance_transactions(self):
        """Test case for get_customer_balance_transactions

        Get the customers balance transactions  # noqa: E501
        """
        pass

    def test_get_feature(self):
        """Test case for get_feature

        Get a feature by id  # noqa: E501
        """
        pass

    def test_get_product(self):
        """Test case for get_product

        Get a single product  # noqa: E501
        """
        pass

    def test_get_subscription(self):
        """Test case for get_subscription

        Get a single billing subscription  # noqa: E501
        """
        pass

    def test_get_subscription_balance_transactions(self):
        """Test case for get_subscription_balance_transactions

        Get the subscription balance transactions  # noqa: E501
        """
        pass

    def test_get_usage_records(self):
        """Test case for get_usage_records

        Get all subscription usage records  # noqa: E501
        """
        pass

    def test_list_billing_accounts(self):
        """Test case for list_billing_accounts

        Get all billing accounts  # noqa: E501
        """
        pass

    def test_list_checkout_sessions(self):
        """Test case for list_checkout_sessions

        list checkout sessions  # noqa: E501
        """
        pass

    def test_list_features(self):
        """Test case for list_features

        Get all features  # noqa: E501
        """
        pass

    def test_list_products(self):
        """Test case for list_products

        Get all products  # noqa: E501
        """
        pass

    def test_list_subscription_features(self):
        """Test case for list_subscription_features

        Get all subscription features  # noqa: E501
        """
        pass

    def test_list_subscriptions(self):
        """Test case for list_subscriptions

        Get all billing subscriptions for a billing account  # noqa: E501
        """
        pass

    def test_list_subscriptions_with_feature(self):
        """Test case for list_subscriptions_with_feature

        Get all subscriptions using feature_id  # noqa: E501
        """
        pass

    def test_new_subscription(self):
        """Test case for new_subscription

        Create a new upstream subscription  # noqa: E501
        """
        pass

    def test_remove_org_from_billing_account(self):
        """Test case for remove_org_from_billing_account

        Remove an org from a billing account  # noqa: E501
        """
        pass

    def test_remove_org_from_billing_subscription(self):
        """Test case for remove_org_from_billing_subscription

        Remove an org from a billing subscription  # noqa: E501
        """
        pass

    def test_replace_billing_account(self):
        """Test case for replace_billing_account

        Create or update a billing account  # noqa: E501
        """
        pass

    def test_replace_feature(self):
        """Test case for replace_feature

        update a Feature  # noqa: E501
        """
        pass

    def test_replace_product(self):
        """Test case for replace_product

        Create or update a product  # noqa: E501
        """
        pass

    def test_replace_subscription(self):
        """Test case for replace_subscription

        Create or update a billing subscription  # noqa: E501
        """
        pass

    def test_update_subscription_cancellation(self):
        """Test case for update_subscription_cancellation

        Update the subscription cancellation detail  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



from unittest.mock import MagicMock

class UsersApiMock:

    def __init__(self):
        self.mock_bulk_approve_requests = MagicMock()
        self.mock_bulk_update_metadata = MagicMock()
        self.mock_create_challenge_method = MagicMock()
        self.mock_create_service_account = MagicMock()
        self.mock_create_support_request = MagicMock()
        self.mock_create_support_request_message = MagicMock()
        self.mock_create_upstream_user_identity = MagicMock()
        self.mock_create_user = MagicMock()
        self.mock_create_user_identity_update = MagicMock()
        self.mock_create_user_metadata = MagicMock()
        self.mock_create_user_request = MagicMock()
        self.mock_delete_challenge_method = MagicMock()
        self.mock_delete_service_account = MagicMock()
        self.mock_delete_support_request = MagicMock()
        self.mock_delete_upstream_user_identity = MagicMock()
        self.mock_delete_user = MagicMock()
        self.mock_delete_user_metadata = MagicMock()
        self.mock_delete_user_request = MagicMock()
        self.mock_get_challenge_method = MagicMock()
        self.mock_get_service_account = MagicMock()
        self.mock_get_support_request = MagicMock()
        self.mock_get_upstream_user_identity = MagicMock()
        self.mock_get_user = MagicMock()
        self.mock_get_user_metadata = MagicMock()
        self.mock_get_user_request = MagicMock()
        self.mock_list_access_requests = MagicMock()
        self.mock_list_all_resource_permissions = MagicMock()
        self.mock_list_all_user_orgs = MagicMock()
        self.mock_list_all_user_roles = MagicMock()
        self.mock_list_challenge_methods = MagicMock()
        self.mock_list_combined_user_details = MagicMock()
        self.mock_list_desktop_access_info = MagicMock()
        self.mock_list_org_user_roles = MagicMock()
        self.mock_list_service_accounts = MagicMock()
        self.mock_list_ssh_access_info = MagicMock()
        self.mock_list_support_requests = MagicMock()
        self.mock_list_upstream_user_identities = MagicMock()
        self.mock_list_user_application_access_info = MagicMock()
        self.mock_list_user_file_share_access_info = MagicMock()
        self.mock_list_user_guid_mapping = MagicMock()
        self.mock_list_user_guids = MagicMock()
        self.mock_list_user_launcher_access_info = MagicMock()
        self.mock_list_user_metadata = MagicMock()
        self.mock_list_user_permissions = MagicMock()
        self.mock_list_user_requests = MagicMock()
        self.mock_list_user_resource_access_info = MagicMock()
        self.mock_list_users = MagicMock()
        self.mock_replace_challenge_method = MagicMock()
        self.mock_replace_service_account = MagicMock()
        self.mock_replace_upstream_user_identity = MagicMock()
        self.mock_replace_user = MagicMock()
        self.mock_replace_user_metadata = MagicMock()
        self.mock_replace_user_request = MagicMock()
        self.mock_replace_user_role = MagicMock()
        self.mock_reset_user_identity = MagicMock()
        self.mock_reset_user_mfa_challenge_methods = MagicMock()
        self.mock_update_support_request = MagicMock()
        self.mock_update_user_request = MagicMock()

    def bulk_approve_requests(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.bulk_approve_requests with MagicMock.
        """
        return self.mock_bulk_approve_requests(self, *args, **kwargs)

    def bulk_update_metadata(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.bulk_update_metadata with MagicMock.
        """
        return self.mock_bulk_update_metadata(self, *args, **kwargs)

    def create_challenge_method(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.create_challenge_method with MagicMock.
        """
        return self.mock_create_challenge_method(self, *args, **kwargs)

    def create_service_account(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.create_service_account with MagicMock.
        """
        return self.mock_create_service_account(self, *args, **kwargs)

    def create_support_request(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.create_support_request with MagicMock.
        """
        return self.mock_create_support_request(self, *args, **kwargs)

    def create_support_request_message(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.create_support_request_message with MagicMock.
        """
        return self.mock_create_support_request_message(self, *args, **kwargs)

    def create_upstream_user_identity(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.create_upstream_user_identity with MagicMock.
        """
        return self.mock_create_upstream_user_identity(self, *args, **kwargs)

    def create_user(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.create_user with MagicMock.
        """
        return self.mock_create_user(self, *args, **kwargs)

    def create_user_identity_update(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.create_user_identity_update with MagicMock.
        """
        return self.mock_create_user_identity_update(self, *args, **kwargs)

    def create_user_metadata(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.create_user_metadata with MagicMock.
        """
        return self.mock_create_user_metadata(self, *args, **kwargs)

    def create_user_request(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.create_user_request with MagicMock.
        """
        return self.mock_create_user_request(self, *args, **kwargs)

    def delete_challenge_method(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.delete_challenge_method with MagicMock.
        """
        return self.mock_delete_challenge_method(self, *args, **kwargs)

    def delete_service_account(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.delete_service_account with MagicMock.
        """
        return self.mock_delete_service_account(self, *args, **kwargs)

    def delete_support_request(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.delete_support_request with MagicMock.
        """
        return self.mock_delete_support_request(self, *args, **kwargs)

    def delete_upstream_user_identity(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.delete_upstream_user_identity with MagicMock.
        """
        return self.mock_delete_upstream_user_identity(self, *args, **kwargs)

    def delete_user(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.delete_user with MagicMock.
        """
        return self.mock_delete_user(self, *args, **kwargs)

    def delete_user_metadata(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.delete_user_metadata with MagicMock.
        """
        return self.mock_delete_user_metadata(self, *args, **kwargs)

    def delete_user_request(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.delete_user_request with MagicMock.
        """
        return self.mock_delete_user_request(self, *args, **kwargs)

    def get_challenge_method(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.get_challenge_method with MagicMock.
        """
        return self.mock_get_challenge_method(self, *args, **kwargs)

    def get_service_account(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.get_service_account with MagicMock.
        """
        return self.mock_get_service_account(self, *args, **kwargs)

    def get_support_request(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.get_support_request with MagicMock.
        """
        return self.mock_get_support_request(self, *args, **kwargs)

    def get_upstream_user_identity(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.get_upstream_user_identity with MagicMock.
        """
        return self.mock_get_upstream_user_identity(self, *args, **kwargs)

    def get_user(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.get_user with MagicMock.
        """
        return self.mock_get_user(self, *args, **kwargs)

    def get_user_metadata(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.get_user_metadata with MagicMock.
        """
        return self.mock_get_user_metadata(self, *args, **kwargs)

    def get_user_request(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.get_user_request with MagicMock.
        """
        return self.mock_get_user_request(self, *args, **kwargs)

    def list_access_requests(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_access_requests with MagicMock.
        """
        return self.mock_list_access_requests(self, *args, **kwargs)

    def list_all_resource_permissions(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_all_resource_permissions with MagicMock.
        """
        return self.mock_list_all_resource_permissions(self, *args, **kwargs)

    def list_all_user_orgs(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_all_user_orgs with MagicMock.
        """
        return self.mock_list_all_user_orgs(self, *args, **kwargs)

    def list_all_user_roles(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_all_user_roles with MagicMock.
        """
        return self.mock_list_all_user_roles(self, *args, **kwargs)

    def list_challenge_methods(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_challenge_methods with MagicMock.
        """
        return self.mock_list_challenge_methods(self, *args, **kwargs)

    def list_combined_user_details(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_combined_user_details with MagicMock.
        """
        return self.mock_list_combined_user_details(self, *args, **kwargs)

    def list_desktop_access_info(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_desktop_access_info with MagicMock.
        """
        return self.mock_list_desktop_access_info(self, *args, **kwargs)

    def list_org_user_roles(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_org_user_roles with MagicMock.
        """
        return self.mock_list_org_user_roles(self, *args, **kwargs)

    def list_service_accounts(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_service_accounts with MagicMock.
        """
        return self.mock_list_service_accounts(self, *args, **kwargs)

    def list_ssh_access_info(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_ssh_access_info with MagicMock.
        """
        return self.mock_list_ssh_access_info(self, *args, **kwargs)

    def list_support_requests(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_support_requests with MagicMock.
        """
        return self.mock_list_support_requests(self, *args, **kwargs)

    def list_upstream_user_identities(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_upstream_user_identities with MagicMock.
        """
        return self.mock_list_upstream_user_identities(self, *args, **kwargs)

    def list_user_application_access_info(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_user_application_access_info with MagicMock.
        """
        return self.mock_list_user_application_access_info(self, *args, **kwargs)

    def list_user_file_share_access_info(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_user_file_share_access_info with MagicMock.
        """
        return self.mock_list_user_file_share_access_info(self, *args, **kwargs)

    def list_user_guid_mapping(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_user_guid_mapping with MagicMock.
        """
        return self.mock_list_user_guid_mapping(self, *args, **kwargs)

    def list_user_guids(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_user_guids with MagicMock.
        """
        return self.mock_list_user_guids(self, *args, **kwargs)

    def list_user_launcher_access_info(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_user_launcher_access_info with MagicMock.
        """
        return self.mock_list_user_launcher_access_info(self, *args, **kwargs)

    def list_user_metadata(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_user_metadata with MagicMock.
        """
        return self.mock_list_user_metadata(self, *args, **kwargs)

    def list_user_permissions(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_user_permissions with MagicMock.
        """
        return self.mock_list_user_permissions(self, *args, **kwargs)

    def list_user_requests(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_user_requests with MagicMock.
        """
        return self.mock_list_user_requests(self, *args, **kwargs)

    def list_user_resource_access_info(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_user_resource_access_info with MagicMock.
        """
        return self.mock_list_user_resource_access_info(self, *args, **kwargs)

    def list_users(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.list_users with MagicMock.
        """
        return self.mock_list_users(self, *args, **kwargs)

    def replace_challenge_method(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.replace_challenge_method with MagicMock.
        """
        return self.mock_replace_challenge_method(self, *args, **kwargs)

    def replace_service_account(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.replace_service_account with MagicMock.
        """
        return self.mock_replace_service_account(self, *args, **kwargs)

    def replace_upstream_user_identity(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.replace_upstream_user_identity with MagicMock.
        """
        return self.mock_replace_upstream_user_identity(self, *args, **kwargs)

    def replace_user(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.replace_user with MagicMock.
        """
        return self.mock_replace_user(self, *args, **kwargs)

    def replace_user_metadata(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.replace_user_metadata with MagicMock.
        """
        return self.mock_replace_user_metadata(self, *args, **kwargs)

    def replace_user_request(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.replace_user_request with MagicMock.
        """
        return self.mock_replace_user_request(self, *args, **kwargs)

    def replace_user_role(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.replace_user_role with MagicMock.
        """
        return self.mock_replace_user_role(self, *args, **kwargs)

    def reset_user_identity(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.reset_user_identity with MagicMock.
        """
        return self.mock_reset_user_identity(self, *args, **kwargs)

    def reset_user_mfa_challenge_methods(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.reset_user_mfa_challenge_methods with MagicMock.
        """
        return self.mock_reset_user_mfa_challenge_methods(self, *args, **kwargs)

    def update_support_request(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.update_support_request with MagicMock.
        """
        return self.mock_update_support_request(self, *args, **kwargs)

    def update_user_request(self, *args, **kwargs):
        """
        This method mocks the original api UsersApi.update_user_request with MagicMock.
        """
        return self.mock_update_user_request(self, *args, **kwargs)




"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2025.04.10
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from agilicus_api.api_client import ApiClient, Endpoint as _Endpoint
from agilicus_api.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from agilicus_api.model.cluster import Cluster
from agilicus_api.model.error_message import ErrorMessage
from agilicus_api.model.list_cluster_response import ListClusterResponse
from agilicus_api.model.list_point_of_presences_response import ListPointOfPresencesResponse
from agilicus_api.model.list_region_response import ListRegionResponse
from agilicus_api.model.point_of_presence import PointOfPresence
from agilicus_api.model.region import Region
from agilicus_api.model.routing_request import RoutingRequest


class RegionsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __add_cluster(
            self,
            cluster,
            **kwargs
        ):
            """Add a Cluster  # noqa: E501

            Adds a new Cluster   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.add_cluster(cluster, async_req=True)
            >>> result = thread.get()

            Args:
                cluster (Cluster):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Cluster
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['cluster'] = \
                cluster
            return self.call_with_http_info(**kwargs)

        if self.add_cluster is None:
            self.add_cluster = _Endpoint(
                settings={
                    'response_type': (Cluster,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/clusters',
                    'operation_id': 'add_cluster',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'cluster',
                    ],
                    'required': [
                        'cluster',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'cluster':
                            (Cluster,),
                    },
                    'attribute_map': {
                    },
                    'location_map': {
                        'cluster': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__add_cluster
            )

        def __add_point_of_presence(
            self,
            point_of_presence,
            **kwargs
        ):
            """Add a point of presence.  # noqa: E501

            Adds a new point of presence.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.add_point_of_presence(point_of_presence, async_req=True)
            >>> result = thread.get()

            Args:
                point_of_presence (PointOfPresence):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                PointOfPresence
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['point_of_presence'] = \
                point_of_presence
            return self.call_with_http_info(**kwargs)

        if self.add_point_of_presence is None:
            self.add_point_of_presence = _Endpoint(
                settings={
                    'response_type': (PointOfPresence,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/point_of_presences',
                    'operation_id': 'add_point_of_presence',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'point_of_presence',
                    ],
                    'required': [
                        'point_of_presence',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'point_of_presence':
                            (PointOfPresence,),
                    },
                    'attribute_map': {
                    },
                    'location_map': {
                        'point_of_presence': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__add_point_of_presence
            )

        def __add_region(
            self,
            region,
            **kwargs
        ):
            """Add a Region  # noqa: E501

            Adds a new Region   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.add_region(region, async_req=True)
            >>> result = thread.get()

            Args:
                region (Region):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Region
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['region'] = \
                region
            return self.call_with_http_info(**kwargs)

        if self.add_region is None:
            self.add_region = _Endpoint(
                settings={
                    'response_type': (Region,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/regions',
                    'operation_id': 'add_region',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'region',
                    ],
                    'required': [
                        'region',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'region':
                            (Region,),
                    },
                    'attribute_map': {
                    },
                    'location_map': {
                        'region': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__add_region
            )

        def __delete_cluster(
            self,
            cluster_id,
            **kwargs
        ):
            """Delete a Cluster  # noqa: E501

            Delete a Cluster  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.delete_cluster(cluster_id, async_req=True)
            >>> result = thread.get()

            Args:
                cluster_id (str): cluster id in path

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['cluster_id'] = \
                cluster_id
            return self.call_with_http_info(**kwargs)

        if self.delete_cluster is None:
            self.delete_cluster = _Endpoint(
                settings={
                    'response_type': None,
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/clusters/{cluster_id}',
                    'operation_id': 'delete_cluster',
                    'http_method': 'DELETE',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'cluster_id',
                    ],
                    'required': [
                        'cluster_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'cluster_id':
                            (str,),
                    },
                    'attribute_map': {
                        'cluster_id': 'cluster_id',
                    },
                    'location_map': {
                        'cluster_id': 'path',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__delete_cluster
            )

        def __delete_point_of_presence(
            self,
            point_of_presence_id,
            **kwargs
        ):
            """Delete a point of presence  # noqa: E501

            Delete a point of presence  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.delete_point_of_presence(point_of_presence_id, async_req=True)
            >>> result = thread.get()

            Args:
                point_of_presence_id (str): point of presence id in path

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['point_of_presence_id'] = \
                point_of_presence_id
            return self.call_with_http_info(**kwargs)

        if self.delete_point_of_presence is None:
            self.delete_point_of_presence = _Endpoint(
                settings={
                    'response_type': None,
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/point_of_presences/{point_of_presence_id}',
                    'operation_id': 'delete_point_of_presence',
                    'http_method': 'DELETE',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'point_of_presence_id',
                    ],
                    'required': [
                        'point_of_presence_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'point_of_presence_id':
                            (str,),
                    },
                    'attribute_map': {
                        'point_of_presence_id': 'point_of_presence_id',
                    },
                    'location_map': {
                        'point_of_presence_id': 'path',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__delete_point_of_presence
            )

        def __delete_region(
            self,
            region_id,
            **kwargs
        ):
            """Delete a Region  # noqa: E501

            Delete a Region  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.delete_region(region_id, async_req=True)
            >>> result = thread.get()

            Args:
                region_id (str): region id in path

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['region_id'] = \
                region_id
            return self.call_with_http_info(**kwargs)

        if self.delete_region is None:
            self.delete_region = _Endpoint(
                settings={
                    'response_type': None,
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/regions/{region_id}',
                    'operation_id': 'delete_region',
                    'http_method': 'DELETE',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'region_id',
                    ],
                    'required': [
                        'region_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'region_id':
                            (str,),
                    },
                    'attribute_map': {
                        'region_id': 'region_id',
                    },
                    'location_map': {
                        'region_id': 'path',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__delete_region
            )

        def __get_cluster(
            self,
            cluster_id,
            **kwargs
        ):
            """Get a Cluster  # noqa: E501

            Get a Cluster  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.get_cluster(cluster_id, async_req=True)
            >>> result = thread.get()

            Args:
                cluster_id (str): cluster id in path

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Cluster
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['cluster_id'] = \
                cluster_id
            return self.call_with_http_info(**kwargs)

        if self.get_cluster is None:
            self.get_cluster = _Endpoint(
                settings={
                    'response_type': (Cluster,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/clusters/{cluster_id}',
                    'operation_id': 'get_cluster',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'cluster_id',
                    ],
                    'required': [
                        'cluster_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'cluster_id':
                            (str,),
                    },
                    'attribute_map': {
                        'cluster_id': 'cluster_id',
                    },
                    'location_map': {
                        'cluster_id': 'path',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__get_cluster
            )

        def __get_point_of_presence(
            self,
            point_of_presence_id,
            **kwargs
        ):
            """Get a point of presence  # noqa: E501

            Get a point of presence  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.get_point_of_presence(point_of_presence_id, async_req=True)
            >>> result = thread.get()

            Args:
                point_of_presence_id (str): point of presence id in path

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                PointOfPresence
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['point_of_presence_id'] = \
                point_of_presence_id
            return self.call_with_http_info(**kwargs)

        if self.get_point_of_presence is None:
            self.get_point_of_presence = _Endpoint(
                settings={
                    'response_type': (PointOfPresence,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/point_of_presences/{point_of_presence_id}',
                    'operation_id': 'get_point_of_presence',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'point_of_presence_id',
                    ],
                    'required': [
                        'point_of_presence_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'point_of_presence_id':
                            (str,),
                    },
                    'attribute_map': {
                        'point_of_presence_id': 'point_of_presence_id',
                    },
                    'location_map': {
                        'point_of_presence_id': 'path',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__get_point_of_presence
            )

        def __get_region(
            self,
            region_id,
            **kwargs
        ):
            """Get a Region  # noqa: E501

            Get a Region  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.get_region(region_id, async_req=True)
            >>> result = thread.get()

            Args:
                region_id (str): region id in path

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Region
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['region_id'] = \
                region_id
            return self.call_with_http_info(**kwargs)

        if self.get_region is None:
            self.get_region = _Endpoint(
                settings={
                    'response_type': (Region,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/regions/{region_id}',
                    'operation_id': 'get_region',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'region_id',
                    ],
                    'required': [
                        'region_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'region_id':
                            (str,),
                    },
                    'attribute_map': {
                        'region_id': 'region_id',
                    },
                    'location_map': {
                        'region_id': 'path',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__get_region
            )

        def __list_clusters(
            self,
            **kwargs
        ):
            """List all Clusters  # noqa: E501

            List all Clusters matching the provided query parameters.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.list_clusters(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                limit (int): limit the number of rows in the response. [optional] if omitted the server will use the default value of 500
                page_at_name (str): Pagination based query with the name as the key. To get the initial entries supply an empty string. On subsequent requests, supply the `page_at_name` field from the list response. . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ListClusterResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        if self.list_clusters is None:
            self.list_clusters = _Endpoint(
                settings={
                    'response_type': (ListClusterResponse,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/clusters',
                    'operation_id': 'list_clusters',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'limit',
                        'page_at_name',
                    ],
                    'required': [],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                        'limit',
                    ]
                },
                root_map={
                    'validations': {
                        ('limit',): {

                            'inclusive_maximum': 500,
                            'inclusive_minimum': 1,
                        },
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'limit':
                            (int,),
                        'page_at_name':
                            (str,),
                    },
                    'attribute_map': {
                        'limit': 'limit',
                        'page_at_name': 'page_at_name',
                    },
                    'location_map': {
                        'limit': 'query',
                        'page_at_name': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__list_clusters
            )

        def __list_point_of_presences(
            self,
            **kwargs
        ):
            """List all regions  # noqa: E501

            List all regions matching the provided query parameters. Perform keyset pagination by setting the page_at_name parameter to the name for the next page to fetch. Set it to `\"\"` to start from the beginning.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.list_point_of_presences(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                limit (int): limit the number of rows in the response. [optional] if omitted the server will use the default value of 500
                name (str): Filters based on whether or not the items in the collection have the given name. . [optional]
                page_at_name (str): Pagination based query with the name as the key. To get the initial entries supply an empty string. On subsequent requests, supply the `page_at_name` field from the list response. . [optional]
                includes_all_tag ([str]): A list of case-sensitive tags to include in the search. Each provided tag must match in order for the item to be returned. . [optional]
                includes_any_tag ([str]): A list of case-sensitive tags to include in the search. If any provided tag matches then the item is returned. . [optional]
                excludes_all_tag ([str]): A list of case-sensitive tags to exclude in the search. If all provided tags match, then the item is not returned. . [optional]
                excludes_any_tag ([str]): A list of case-sensitive tags to exclude in the search. If any provided tag matches, then the item is not returned. . [optional]
                cluster_name (str): query for specific cluster. [optional]
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ListPointOfPresencesResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        if self.list_point_of_presences is None:
            self.list_point_of_presences = _Endpoint(
                settings={
                    'response_type': (ListPointOfPresencesResponse,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/point_of_presences',
                    'operation_id': 'list_point_of_presences',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'limit',
                        'name',
                        'page_at_name',
                        'includes_all_tag',
                        'includes_any_tag',
                        'excludes_all_tag',
                        'excludes_any_tag',
                        'cluster_name',
                        'org_id',
                    ],
                    'required': [],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                        'limit',
                    ]
                },
                root_map={
                    'validations': {
                        ('limit',): {

                            'inclusive_maximum': 500,
                            'inclusive_minimum': 1,
                        },
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'limit':
                            (int,),
                        'name':
                            (str,),
                        'page_at_name':
                            (str,),
                        'includes_all_tag':
                            ([str],),
                        'includes_any_tag':
                            ([str],),
                        'excludes_all_tag':
                            ([str],),
                        'excludes_any_tag':
                            ([str],),
                        'cluster_name':
                            (str,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'limit': 'limit',
                        'name': 'name',
                        'page_at_name': 'page_at_name',
                        'includes_all_tag': 'includes_all_tag',
                        'includes_any_tag': 'includes_any_tag',
                        'excludes_all_tag': 'excludes_all_tag',
                        'excludes_any_tag': 'excludes_any_tag',
                        'cluster_name': 'cluster_name',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'limit': 'query',
                        'name': 'query',
                        'page_at_name': 'query',
                        'includes_all_tag': 'query',
                        'includes_any_tag': 'query',
                        'excludes_all_tag': 'query',
                        'excludes_any_tag': 'query',
                        'cluster_name': 'query',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                        'includes_all_tag': 'multi',
                        'includes_any_tag': 'multi',
                        'excludes_all_tag': 'multi',
                        'excludes_any_tag': 'multi',
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__list_point_of_presences
            )

        def __list_regions(
            self,
            **kwargs
        ):
            """List all regions  # noqa: E501

            List all Regions matching the provided query parameters.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.list_regions(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                limit (int): limit the number of rows in the response. [optional] if omitted the server will use the default value of 500
                page_at_name (str): Pagination based query with the name as the key. To get the initial entries supply an empty string. On subsequent requests, supply the `page_at_name` field from the list response. . [optional]
                org_id (str): Organisation Unique identifier. [optional]
                cluster_name (str): query for specific cluster. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ListRegionResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        if self.list_regions is None:
            self.list_regions = _Endpoint(
                settings={
                    'response_type': (ListRegionResponse,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/regions',
                    'operation_id': 'list_regions',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'limit',
                        'page_at_name',
                        'org_id',
                        'cluster_name',
                    ],
                    'required': [],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                        'limit',
                    ]
                },
                root_map={
                    'validations': {
                        ('limit',): {

                            'inclusive_maximum': 500,
                            'inclusive_minimum': 1,
                        },
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'limit':
                            (int,),
                        'page_at_name':
                            (str,),
                        'org_id':
                            (str,),
                        'cluster_name':
                            (str,),
                    },
                    'attribute_map': {
                        'limit': 'limit',
                        'page_at_name': 'page_at_name',
                        'org_id': 'org_id',
                        'cluster_name': 'cluster_name',
                    },
                    'location_map': {
                        'limit': 'query',
                        'page_at_name': 'query',
                        'org_id': 'query',
                        'cluster_name': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__list_regions
            )

        def __replace_cluster(
            self,
            cluster_id,
            **kwargs
        ):
            """update a Cluster  # noqa: E501

            update a Cluster  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.replace_cluster(cluster_id, async_req=True)
            >>> result = thread.get()

            Args:
                cluster_id (str): cluster id in path

            Keyword Args:
                cluster (Cluster): [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Cluster
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['cluster_id'] = \
                cluster_id
            return self.call_with_http_info(**kwargs)

        if self.replace_cluster is None:
            self.replace_cluster = _Endpoint(
                settings={
                    'response_type': (Cluster,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/clusters/{cluster_id}',
                    'operation_id': 'replace_cluster',
                    'http_method': 'PUT',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'cluster_id',
                        'cluster',
                    ],
                    'required': [
                        'cluster_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'cluster_id':
                            (str,),
                        'cluster':
                            (Cluster,),
                    },
                    'attribute_map': {
                        'cluster_id': 'cluster_id',
                    },
                    'location_map': {
                        'cluster_id': 'path',
                        'cluster': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__replace_cluster
            )

        def __replace_point_of_presence(
            self,
            point_of_presence_id,
            **kwargs
        ):
            """update a point of presence  # noqa: E501

            update a point of presence  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.replace_point_of_presence(point_of_presence_id, async_req=True)
            >>> result = thread.get()

            Args:
                point_of_presence_id (str): point of presence id in path

            Keyword Args:
                point_of_presence (PointOfPresence): [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                PointOfPresence
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['point_of_presence_id'] = \
                point_of_presence_id
            return self.call_with_http_info(**kwargs)

        if self.replace_point_of_presence is None:
            self.replace_point_of_presence = _Endpoint(
                settings={
                    'response_type': (PointOfPresence,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/point_of_presences/{point_of_presence_id}',
                    'operation_id': 'replace_point_of_presence',
                    'http_method': 'PUT',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'point_of_presence_id',
                        'point_of_presence',
                    ],
                    'required': [
                        'point_of_presence_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'point_of_presence_id':
                            (str,),
                        'point_of_presence':
                            (PointOfPresence,),
                    },
                    'attribute_map': {
                        'point_of_presence_id': 'point_of_presence_id',
                    },
                    'location_map': {
                        'point_of_presence_id': 'path',
                        'point_of_presence': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__replace_point_of_presence
            )

        def __replace_region(
            self,
            region_id,
            **kwargs
        ):
            """update a Region  # noqa: E501

            update a Region  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.replace_region(region_id, async_req=True)
            >>> result = thread.get()

            Args:
                region_id (str): region id in path

            Keyword Args:
                region (Region): [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Region
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['region_id'] = \
                region_id
            return self.call_with_http_info(**kwargs)

        if self.replace_region is None:
            self.replace_region = _Endpoint(
                settings={
                    'response_type': (Region,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/regions/{region_id}',
                    'operation_id': 'replace_region',
                    'http_method': 'PUT',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'region_id',
                        'region',
                    ],
                    'required': [
                        'region_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'region_id':
                            (str,),
                        'region':
                            (Region,),
                    },
                    'attribute_map': {
                        'region_id': 'region_id',
                    },
                    'location_map': {
                        'region_id': 'path',
                        'region': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__replace_region
            )

        def __routing_request(
            self,
            routing_request,
            **kwargs
        ):
            """Request routing  # noqa: E501

            Request routing for a particular list of ip_addresses   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.routing_request(routing_request, async_req=True)
            >>> result = thread.get()

            Args:
                routing_request (RoutingRequest):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                RoutingRequest
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['routing_request'] = \
                routing_request
            return self.call_with_http_info(**kwargs)

        if self.routing_request is None:
            self.routing_request = _Endpoint(
                settings={
                    'response_type': (RoutingRequest,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/routing_request',
                    'operation_id': 'routing_request',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'routing_request',
                    ],
                    'required': [
                        'routing_request',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'routing_request':
                            (RoutingRequest,),
                    },
                    'attribute_map': {
                    },
                    'location_map': {
                        'routing_request': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__routing_request
            )

    add_cluster = None 
    add_point_of_presence = None 
    add_region = None 
    delete_cluster = None 
    delete_point_of_presence = None 
    delete_region = None 
    get_cluster = None 
    get_point_of_presence = None 
    get_region = None 
    list_clusters = None 
    list_point_of_presences = None 
    list_regions = None 
    replace_cluster = None 
    replace_point_of_presence = None 
    replace_region = None 
    routing_request = None 



from unittest.mock import MagicMock

class MetricsApiMock:

    def __init__(self):
        self.mock_list_active_users = MagicMock()
        self.mock_list_top_users = MagicMock()

    def list_active_users(self, *args, **kwargs):
        """
        This method mocks the original api MetricsApi.list_active_users with MagicMock.
        """
        return self.mock_list_active_users(self, *args, **kwargs)

    def list_top_users(self, *args, **kwargs):
        """
        This method mocks the original api MetricsApi.list_top_users with MagicMock.
        """
        return self.mock_list_top_users(self, *args, **kwargs)




from unittest.mock import MagicMock

class HostsApiMock:

    def __init__(self):
        self.mock_add_host = MagicMock()
        self.mock_add_host_bundle = MagicMock()
        self.mock_add_host_label = MagicMock()
        self.mock_delete_host = MagicMock()
        self.mock_delete_host_bundle = MagicMock()
        self.mock_delete_host_label = MagicMock()
        self.mock_get_host = MagicMock()
        self.mock_get_host_bundle = MagicMock()
        self.mock_get_host_label = MagicMock()
        self.mock_list_host_bundles = MagicMock()
        self.mock_list_host_labels = MagicMock()
        self.mock_list_host_orgs = MagicMock()
        self.mock_list_hosts = MagicMock()
        self.mock_replace_host = MagicMock()
        self.mock_replace_host_bundle = MagicMock()

    def add_host(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.add_host with MagicMock.
        """
        return self.mock_add_host(self, *args, **kwargs)

    def add_host_bundle(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.add_host_bundle with MagicMock.
        """
        return self.mock_add_host_bundle(self, *args, **kwargs)

    def add_host_label(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.add_host_label with MagicMock.
        """
        return self.mock_add_host_label(self, *args, **kwargs)

    def delete_host(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.delete_host with MagicMock.
        """
        return self.mock_delete_host(self, *args, **kwargs)

    def delete_host_bundle(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.delete_host_bundle with MagicMock.
        """
        return self.mock_delete_host_bundle(self, *args, **kwargs)

    def delete_host_label(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.delete_host_label with MagicMock.
        """
        return self.mock_delete_host_label(self, *args, **kwargs)

    def get_host(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.get_host with MagicMock.
        """
        return self.mock_get_host(self, *args, **kwargs)

    def get_host_bundle(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.get_host_bundle with MagicMock.
        """
        return self.mock_get_host_bundle(self, *args, **kwargs)

    def get_host_label(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.get_host_label with MagicMock.
        """
        return self.mock_get_host_label(self, *args, **kwargs)

    def list_host_bundles(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.list_host_bundles with MagicMock.
        """
        return self.mock_list_host_bundles(self, *args, **kwargs)

    def list_host_labels(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.list_host_labels with MagicMock.
        """
        return self.mock_list_host_labels(self, *args, **kwargs)

    def list_host_orgs(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.list_host_orgs with MagicMock.
        """
        return self.mock_list_host_orgs(self, *args, **kwargs)

    def list_hosts(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.list_hosts with MagicMock.
        """
        return self.mock_list_hosts(self, *args, **kwargs)

    def replace_host(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.replace_host with MagicMock.
        """
        return self.mock_replace_host(self, *args, **kwargs)

    def replace_host_bundle(self, *args, **kwargs):
        """
        This method mocks the original api HostsApi.replace_host_bundle with MagicMock.
        """
        return self.mock_replace_host_bundle(self, *args, **kwargs)




from .. import context
from agilicus.agilicus_api import (
    NetworkPort,
    NetworkPortRange,
    NetworkServiceConfig,
)

from agilicus import agilicus_api

from ..input_helpers import build_updated_model
from ..input_helpers import update_org_from_input_or_ctx
from ..input_helpers import strip_none
from ..output.table import (
    spec_column,
    format_table,
    metadata_column,
)
from ..resource_helpers import map_resource_published

PROTOCOLS = ["postgresql"]


def list_database_resources(ctx, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    params = strip_none(kwargs)
    query_results = apiclient.app_services_api.list_database_resources(**params)
    return query_results.database_resources


def add_database_resource(ctx, port, name, data_source_name, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    if data_source_name is None:
        data_source_name = name

    spec = agilicus_api.DatabaseResourceSpec(
        name=name, data_source_name=data_source_name, **strip_none(kwargs)
    )
    if port is not None:
        ports = NetworkPortRange(port=NetworkPort(str(port)))
        spec["config"] = NetworkServiceConfig(ports=[ports])
    model = agilicus_api.DatabaseResource(spec=spec)

    return apiclient.app_services_api.create_database_resource(model).to_dict()


def _get_database_resource(ctx, apiclient, resource_id, **kwargs):
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    return apiclient.app_services_api.get_database_resource(resource_id, **kwargs)


def show_database_resource(ctx, resource_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return _get_database_resource(ctx, apiclient, resource_id, **kwargs).to_dict()


def delete_database_resource(ctx, resource_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    return apiclient.app_services_api.delete_database_resource(resource_id, **kwargs)


def update_database_resource(
    ctx,
    resource_id,
    port,
    published,
    runtime_parameters,
    replace_runtime_parameters,
    **kwargs,
):
    apiclient = context.get_apiclient_from_ctx(ctx)
    get_args = {}
    update_org_from_input_or_ctx(get_args, ctx, **kwargs)
    mapping = _get_database_resource(ctx, apiclient, resource_id, **get_args)

    # check_type=False works around nested types not deserializing correctly
    mapping.spec = build_updated_model(
        agilicus_api.DatabaseResourceSpec, mapping.spec, kwargs, check_type=False
    )

    runtime_parameters = list(runtime_parameters or [])

    if not replace_runtime_parameters:
        runtime_parameters = (mapping.spec.runtime_parameters or []) + runtime_parameters

    mapping.spec.runtime_parameters = runtime_parameters

    if port is not None:
        ports = NetworkPortRange(port=NetworkPort(str(port)))
        mapping.spec["config"] = NetworkServiceConfig(ports=[ports])

    mapping = map_resource_published(mapping, published)

    return apiclient.app_services_api.replace_database_resource(
        resource_id, database_resource=mapping
    ).to_dict()


def format_database_as_text(ctx, resources):
    columns = [
        metadata_column("id"),
        spec_column("org_id"),
        spec_column("name"),
        spec_column("address"),
        spec_column("database_protocol"),
        spec_column("data_source_name"),
        spec_column("connector_id"),
        spec_column("config"),
    ]

    return format_table(ctx, resources, columns)



"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2025.04.10
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from agilicus_api.api_client import ApiClient, Endpoint as _Endpoint
from agilicus_api.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from agilicus_api.model.error_message import ErrorMessage
from agilicus_api.model.list_policy_template_instances_response import ListPolicyTemplateInstancesResponse
from agilicus_api.model.object_type import ObjectType
from agilicus_api.model.policy_template_instance import PolicyTemplateInstance


class PolicyTemplatesApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __create_policy_template_instance(
            self,
            policy_template_instance,
            **kwargs
        ):
            """Add a PolicyTemplateInstance  # noqa: E501

            Adds a new PolicyTemplateInstance. PolicyTemplates must have unique names within an org for a particular type of template. If the name is not unique, a 409 will be returned.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.create_policy_template_instance(policy_template_instance, async_req=True)
            >>> result = thread.get()

            Args:
                policy_template_instance (PolicyTemplateInstance):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                PolicyTemplateInstance
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['policy_template_instance'] = \
                policy_template_instance
            return self.call_with_http_info(**kwargs)

        if self.create_policy_template_instance is None:
            self.create_policy_template_instance = _Endpoint(
                settings={
                    'response_type': (PolicyTemplateInstance,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/policy_template_instances',
                    'operation_id': 'create_policy_template_instance',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'policy_template_instance',
                    ],
                    'required': [
                        'policy_template_instance',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'policy_template_instance':
                            (PolicyTemplateInstance,),
                    },
                    'attribute_map': {
                    },
                    'location_map': {
                        'policy_template_instance': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__create_policy_template_instance
            )

        def __delete_policy_template_instance(
            self,
            instance_id,
            **kwargs
        ):
            """Delete a PolicyTemplateInstance  # noqa: E501

            Delete a PolicyTemplateInstance  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.delete_policy_template_instance(instance_id, async_req=True)
            >>> result = thread.get()

            Args:
                instance_id (str): The id of the instance to manipulate/get

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['instance_id'] = \
                instance_id
            return self.call_with_http_info(**kwargs)

        if self.delete_policy_template_instance is None:
            self.delete_policy_template_instance = _Endpoint(
                settings={
                    'response_type': None,
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/policy_template_instances/{instance_id}',
                    'operation_id': 'delete_policy_template_instance',
                    'http_method': 'DELETE',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'instance_id',
                        'org_id',
                    ],
                    'required': [
                        'instance_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'instance_id':
                            (str,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'instance_id': 'instance_id',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'instance_id': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__delete_policy_template_instance
            )

        def __get_policy_template_instance(
            self,
            instance_id,
            **kwargs
        ):
            """Get a PolicyTemplateInstance  # noqa: E501

            Get a PolicyTemplateInstance  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.get_policy_template_instance(instance_id, async_req=True)
            >>> result = thread.get()

            Args:
                instance_id (str): The id of the instance to manipulate/get

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                PolicyTemplateInstance
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['instance_id'] = \
                instance_id
            return self.call_with_http_info(**kwargs)

        if self.get_policy_template_instance is None:
            self.get_policy_template_instance = _Endpoint(
                settings={
                    'response_type': (PolicyTemplateInstance,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/policy_template_instances/{instance_id}',
                    'operation_id': 'get_policy_template_instance',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'instance_id',
                        'org_id',
                    ],
                    'required': [
                        'instance_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'instance_id':
                            (str,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'instance_id': 'instance_id',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'instance_id': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__get_policy_template_instance
            )

        def __list_policy_template_instances(
            self,
            **kwargs
        ):
            """List all standalone policy_templates  # noqa: E501

            List all PolicyTemplateInstances matching the provided query parameters.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.list_policy_template_instances(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                limit (int): limit the number of rows in the response. [optional] if omitted the server will use the default value of 500
                name (str): Filters based on whether or not the items in the collection have the given name. . [optional]
                include_invalid (bool): Includes invalid templates so that they may be cleaned up . [optional]
                template_type (str): filters based on the template type . [optional]
                template_types ([str]): filters based on a list of template types. . [optional]
                object_id (str): search by object id. [optional]
                object_type (ObjectType): An object type. [optional]
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ListPolicyTemplateInstancesResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        if self.list_policy_template_instances is None:
            self.list_policy_template_instances = _Endpoint(
                settings={
                    'response_type': (ListPolicyTemplateInstancesResponse,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/policy_template_instances',
                    'operation_id': 'list_policy_template_instances',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'limit',
                        'name',
                        'include_invalid',
                        'template_type',
                        'template_types',
                        'object_id',
                        'object_type',
                        'org_id',
                    ],
                    'required': [],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                        'limit',
                    ]
                },
                root_map={
                    'validations': {
                        ('limit',): {

                            'inclusive_maximum': 500,
                            'inclusive_minimum': 1,
                        },
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'limit':
                            (int,),
                        'name':
                            (str,),
                        'include_invalid':
                            (bool,),
                        'template_type':
                            (str,),
                        'template_types':
                            ([str],),
                        'object_id':
                            (str,),
                        'object_type':
                            (ObjectType,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'limit': 'limit',
                        'name': 'name',
                        'include_invalid': 'include_invalid',
                        'template_type': 'template_type',
                        'template_types': 'template_types',
                        'object_id': 'object_id',
                        'object_type': 'object_type',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'limit': 'query',
                        'name': 'query',
                        'include_invalid': 'query',
                        'template_type': 'query',
                        'template_types': 'query',
                        'object_id': 'query',
                        'object_type': 'query',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                        'template_types': 'multi',
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__list_policy_template_instances
            )

        def __replace_policy_template_instance(
            self,
            instance_id,
            **kwargs
        ):
            """update a PolicyTemplateInstance  # noqa: E501

            update a PolicyTemplateInstance  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.replace_policy_template_instance(instance_id, async_req=True)
            >>> result = thread.get()

            Args:
                instance_id (str): The id of the instance to manipulate/get

            Keyword Args:
                policy_template_instance (PolicyTemplateInstance): [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                PolicyTemplateInstance
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['instance_id'] = \
                instance_id
            return self.call_with_http_info(**kwargs)

        if self.replace_policy_template_instance is None:
            self.replace_policy_template_instance = _Endpoint(
                settings={
                    'response_type': (PolicyTemplateInstance,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/policy_template_instances/{instance_id}',
                    'operation_id': 'replace_policy_template_instance',
                    'http_method': 'PUT',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'instance_id',
                        'policy_template_instance',
                    ],
                    'required': [
                        'instance_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'instance_id':
                            (str,),
                        'policy_template_instance':
                            (PolicyTemplateInstance,),
                    },
                    'attribute_map': {
                        'instance_id': 'instance_id',
                    },
                    'location_map': {
                        'instance_id': 'path',
                        'policy_template_instance': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__replace_policy_template_instance
            )

    create_policy_template_instance = None 
    delete_policy_template_instance = None 
    get_policy_template_instance = None 
    list_policy_template_instances = None 
    replace_policy_template_instance = None 



import agilicus

from agilicus.agilicus_api import (
    ResourceConfig,
)


def map_resource_published(mapping, published):
    if published is not None:
        config = mapping.spec.resource_config
        if config is None:
            config = ResourceConfig()
            mapping.spec.resource_config = config
        mapping.spec.resource_config["published"] = published
    return mapping


def add_display_info_icon(info_container, uri, purpose, height_px, width_px):
    display_info = info_container.display_info
    if display_info is None:
        display_info = agilicus.DisplayInfo(icons=[])
        info_container.display_info = display_info

    purposes = [agilicus.IconPurpose(p) for p in purpose]
    new_icon = agilicus.Icon(uri=uri, purposes=purposes)
    if height_px is not None and width_px is not None:
        new_icon.dimensions = agilicus.IconDimensions(width=width_px, height=height_px)
    display_info.icons.append(new_icon)


def delete_display_info_icon(info_container, uri):
    if not info_container:
        return False

    display_info = info_container.display_info
    if not info_container:
        return False

    icons = [icon for icon in display_info.icons if icon.uri != uri]
    display_info.icons = icons
    return True
